; Text Screen Dumping module for TSR.

MOV WORD BX, [CEInDOS_Offset]    ; Skips memory dumping if either the critical error or InDOS flag are set.
MOV WORD ES, [CEInDOS_Segment]   ;
ES                               ;
CMP BYTE [BX - 0x01], 0x00       ;
JNE Done                         ;
ES                               ;
CMP BYTE [BX], 0x00              ;
JNE Done                         ;

IN AL, 0x60             ; Skips text screen dumping unless the F12 key is being pressed.
CMP AL, 0x58            ;
JNE Done                ;

CMP BYTE [Busy], 0x00   ; Check whether a dump is already in progress.
JNE Done                ;

MOV BYTE [Busy], 0x01   ; Sets the flag indicating a dump is progress.

MOV AH, 0x3C            ; Creates the output file.
MOV CX, 0x00            ;
LEA DX, OutputFile      ;
INT 0x21                ;
JC Done                 ;

MOV BX, AX              ; Closes the newly created output file.
MOV AH, 0x3E            ;
INT 21h                 ;
JC Done                 ;

MOV AX, 0x3D01          ; Opens the output file for writing.
LEA DX, OutputFile      ;
INT 0x21                ;
JC Done                 ;

MOV BX, AX              ; Retrieves the filehandle.

MOV AX, DS   ; Saves the current data segment.
MOV ES, AX   ;

MOV AX, 0xB800              ; Sets the memory location (text screen video memory) to be written to the output file.
MOV DS, AX                  ;

MOV DX, 0x0000              ; Sets the offset of the first character on the screen.

Dump:
   MOV AH, 0x40                ; Writes the memory block to the output file.
   MOV CX, 0x01                ;
   INT 0x21                    ;
   JC Done                     ;

   CMP DX, 0x0FA0              ; Checks whether the last character on the screen has been reached.
   JAE DumpFinished            ;

   ADD DX, 0x0002              ; Moves to the next character on the screen.
JMP Dump

DumpFinished:
MOV AH, 0x3E          ; Closes the output file.
INT 21h               ;
JMP Done

Address DW 0x0000
Busy DB 0x00
CEInDOS_Offset DW 0x0000
CEInDOS_Segment DW 0x0000
OutputFile DB "ScrnDump.txt", 0x00

Done:
MOV BYTE [Busy], 0x00   ; Clears the flag indicating a dump is progress.
